@@ -3,7 +3,7 @@ package Assert::Refute;
 use 5.006;
 use strict;
 use warnings;
-our $VERSION = '0.16';
+our $VERSION = '0.17';
 
 =head1 NAME
 
@@ -159,7 +159,7 @@ my @core  = qw(
     contract refute_these try_refute
     refute subcontract contract_is current_contract
 );
-my @extra = qw( refute_and_report assert_refute );
+my @extra = qw( assert_refute refute_and_report refute_invariant );
 
 our @ISA = qw(Exporter);
 our @EXPORT = @core;
@@ -180,7 +180,11 @@ $NDEBUG = $ENV{NDEBUG} unless defined $NDEBUG;
 
 sub import {
     my $class = shift;
-    my (%conf, @exp, $need_conf);
+
+    # NOTE ugly hack - allow for 'use Assert::Refute {}'
+    # this should not be needed with new assert_refute / refute_and_report API
+    my %conf = ( on_fail => 'skip' );
+    my (@exp, $need_conf);
     foreach (@_) {
         if (ref $_ eq 'HASH') {
             %conf = (%conf, %$_);
@@ -203,17 +207,26 @@ my %known_callback = (
     skip => '',
     carp => sub {
         my $report = shift;
-        carp $report->get_tap
-            .($report->is_passing ? "Contract passed" : "Contract failed");
+        warn $report->get_tap . _report_mess( $report );
     },
     croak => sub {
         my $report = shift;
-        croak $report->get_tap
-            .($report->is_passing ? "Contract passed" : "Contract failed");
+        die $report->get_tap . _report_mess( $report );
     },
 );
+
+# TODO maybe public method in Report?
+sub _report_mess {
+    my $report = shift;
+
+    my $state = $report->is_passing ? "passed" : "failed";
+    my $title = $report->get_title;
+    my $str   = $title ? "Contract '$title' $state" : "Contract $state";
+    return Carp::shortmess($str);
+};
+
 my %default_conf = (
-    on_fail => 'skip',
+    on_fail => 'carp',
     on_pass => 'skip',
 );
 
@@ -251,7 +264,7 @@ sub try_refute(&;@) { ## no critic # need prototype
     # Should a missing config even happen? Ok, play defensively...
     my $conf = $CALLER_CONF{+caller};
     if( !$conf ) {
-        $conf = __PACKAGE__->configure( { on_fail => 'carp' }, scalar caller );
+        $conf = __PACKAGE__->configure( {}, scalar caller );
     };
     return $conf->{skip_all} if exists $conf->{skip_all};
 
@@ -305,7 +318,35 @@ or C<try_refute>.
 =cut
 
 sub assert_refute(&;@) { ## no critic # need prototype
-    my ( $block, @arg ) = @_;
+    unshift @_, '';
+
+    goto &_real_assert;
+};
+
+=head2 refute_invariant "name" => sub { ... }
+
+A named runtime assertion.
+
+Exactly as above, except that a title is added to the report object
+which will be appended to the emitted warning/error (if any).
+
+Title can be queried via C<get_title> method in report object.
+
+B<[EXPERIMENTAL]>. Name and meaning may change in the future.
+
+=cut
+
+sub refute_invariant(@) { ## no critic # need prototype
+    my ( $name, $block, @arg ) = @_;
+
+    croak q{Usage: refute_invariant "name" => sub { ... }"}
+        unless $name and ref $block eq 'CODE';
+
+    goto &_real_assert;
+};
+
+sub _real_assert {
+    my ( $name, $block ) = @_;
 
     # Should a missing config even happen? Ok, play defensively...
     my $conf = $CALLER_CONF{+caller};
@@ -315,7 +356,7 @@ sub assert_refute(&;@) { ## no critic # need prototype
     };
     return $conf->{skip_all} if exists $conf->{skip_all};
 
-    my $report = $conf->{driver}->new->do_run($block);
+    my $report = $conf->{driver}->new->set_title($name)->do_run($block);
 
     # perform whatever action is needed
     my $callback = $conf->{ $report->is_passing ? "on_pass" : "on_fail" };
@@ -635,6 +676,31 @@ sub configure {
     $CALLER_CONF{$caller} = $conf;
 };
 
+=head2 configure_global( \%global_defaults )
+
+Set a global configuration to be used as default.
+
+This can be used e.g. to set an assertion failure callback throughout
+a big project.
+
+=cut
+
+sub configure_global {
+    my ($class, $conf) = @_;
+
+    croak "Usage: $class->configure_global( \\%hash )"
+        unless ref $conf eq 'HASH';
+
+    my @extra = grep { !$conf_known{$_} } keys %$conf;
+    croak "$class->configure_global: unknown parameters (@extra)"
+        if @extra;
+
+    $default_conf{$_} = $conf->{$_}
+        for keys %$conf;
+
+    return $class; # oh really?
+};
+
 =head2 get_config
 
 Returns configuration from above, initializing with defaults if needed.
