@@ -3,7 +3,7 @@ package Assert::Refute::Report;
 use 5.006;
 use strict;
 use warnings;
-our $VERSION = '0.16';
+our $VERSION = '0.17';
 
 =head1 NAME
 
@@ -91,8 +91,20 @@ and the report will become unconditionally failing.
 
 =cut
 
+my %allow_plan;
+$allow_plan{$_}++ for qw( tests skip_all title );
+
 sub plan {
-    my ($self, $todo, @args) = @_;
+    my $self = shift;
+    $self->_croak("Odd number of arguments in plan()")
+        if @_ % 2;
+    my %args = @_;
+
+    my @extra = grep { !$allow_plan{$_} } keys %args;
+    $self->_croak( "Unknown options to plan(): ".join ",", sort @extra )
+        if @extra;
+    $self->_croak( "Useless use of plan() without arguments" )
+        unless %args;
 
     $self->_croak( $ERROR_DONE )
         if $self->{done};
@@ -102,18 +114,19 @@ sub plan {
     $self->_croak( "plan(): testing already started" )
         if $self->{count} > 0;
 
-    if ($todo eq 'tests') {
-        $self->_croak( "plan(): usage: plan tests => n")
-            unless @args == 1 and defined $args[0] and $args[0] =~ /^\d+$/;
-        $self->{plan_tests} = $args[0];
-    } elsif ($todo eq 'skip_all') {
-        $self->_croak( "plan(): usage: plan skip_all => reason")
-            unless @args == 1 and defined $args[0] and length $args[0];
-        $self->{plan_skip} = $args[0];
+    if ($args{skip_all}) {
+        $self->{plan_skip} = $args{skip_all};
         $self->{plan_tests} = 0;
         # TODO should we lock report?
-    } else {
-        $self->_croak( "Unknown 'plan $todo ...' command" );
+    } elsif (defined $args{tests}) {
+        $self->_croak( "plan(): usage: plan tests => n")
+            unless $args{tests} =~ /^[0-9]+$/;
+        # TODO should we forbid tests => 0 w/o a reason?
+        $self->{plan_tests} = int $args{tests};
+    };
+
+    if ($args{title}) {
+        $self->set_title( $args{title} );
     };
 
     return $self;
@@ -274,6 +287,28 @@ sub set_context {
     return $self;
 };
 
+=head3 set_title
+
+Set the a contract title
+that briefly explains what we are trying to prove, and why.
+
+See also L</get_title>.
+
+B<[EXPERIMENTAL]>. Name and meaning may change in the future.
+
+=cut
+
+# TODO setter
+sub set_title {
+    my ($self, $str) = @_;
+
+    $self->_croak( $ERROR_DONE )
+        if $self->{done};
+
+    $self->{title} = $str;
+    return $self;
+};
+
 =head2 TESTING PRIMITIVES
 
 L<Assert::Refute> comes with a set of basic checks
@@ -659,6 +694,22 @@ sub get_sign {
     return join '', @t, $d;
 };
 
+=head3 get_title
+
+Returns the contract title
+that briefly explains what we are trying to prove, and why.
+
+See also L</set_title>.
+
+B<[EXPERIMENTAL]>. Name and meaning may change in the future.
+
+=cut
+
+# TODO Dumb getter
+sub get_title {
+    return $_[0]->{title};
+};
+
 =head2 DEVELOPMENT PRIMITIVES
 
 Generally one should not touch these methods unless
@@ -844,6 +895,26 @@ sub get_parent {
     return $_[0]->{parent};
 };
 
+=head2 get_depth
+
+Returns 0 is there is no parent, or parent's depth + 1.
+This of this as "this contract's indentation level".
+
+B<EXPERIMENTAL>. Name and meaning MAY change in the future.
+
+=cut
+
+sub get_depth {
+    my $self = shift;
+
+    if (!exists $self->{depth}) {
+        my $parent = $self->get_parent;
+        $self->{depth} = $parent ? $parent->get_depth + 1 : 0;
+    };
+
+    return $self->{depth};
+};
+
 sub _croak {
     my ($self, $mess) = @_;
 
